from typing import Any, final

from cow.errors.exception import VLException
from cow.pydantic.loaders import loadDataFromJson
from cow.web.request import VLRequest
from luna3.client import Client as _Client
from sanic import Sanic
from vlutils.structures.pydantic import BaseModel

from .base_application import BaseAppCtx
from .exceptions import BaseLambdaException
from .monitoring import MonitoringPoint


class BaseLambdaRequest(VLRequest[BaseAppCtx]):
    """
    Luna-configurator request
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.accountId = self.headers.get("Luna-Account-Id")
        self.userCtx = self.app.ctx.userCtx

    @property
    def json(self) -> int | float | list | dict[str, Any]:
        """Get json from request"""
        return super().json

    def getBody(self) -> bytes:
        """Get body from request"""
        return super().body

    def sendToMonitoring(self, points: list[MonitoringPoint]) -> None:
        """
        Send point to monitoring

        Args:
            points: points
        """
        if self.app.ctx.serviceConfig.monitoring.sendData:
            self.app.ctx.monitoring.flushPoints(points)


class ClientsKwargsMixin:
    """Luna services clients kwargs mixin"""

    _luna3Client: _Client
    accountId: str
    app: Sanic

    @property
    def _session(self):
        if hasattr(self.app, "ctx"):
            return self.app.ctx.luna3session
        return self.app.luna3session

    @property
    def _serviceConfig(self):
        if hasattr(self.app, "ctx"):
            return self.app.ctx.serviceConfig
        return self.app.serviceConfig

    def _getFacesClientKwargs(self) -> dict:
        """Get faces clients kwargs"""
        return {
            "facesClient": self._session.facesSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getEventsClientKwargs(self) -> dict | None:
        """Get events clients kwargs"""
        if self._session.eventSession:
            return {
                "eventsClient": self._session.eventSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    def _getLPMClientKwargs(self) -> dict:
        """Get lpn clients kwargs"""
        return {
            "lpmClient": self._session.pythonMatcherSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getLVAClientKwargs(self) -> dict | None:
        """Get lpn clients kwargs"""
        if self._session.videoAgentSession:
            return {
                "lvaClient": self._session.videoAgentSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    def _getFacesSamplesClientKwargs(self) -> dict | None:
        """Get faces samples clients kwargs"""
        if self._session.faceSampleStoreSession:
            return {
                "lisClient": self._session.faceSampleStoreSession.getClient(self.requestId),
                "accountId": self.accountId,
                "bucket": self._serviceConfig.faceSamplesStorage.bucket,
            }

    def _getBodySamplesClientKwargs(self) -> dict | None:
        """Get body samples clients kwargs"""
        if self._session.bodySampleStoreSession:
            return {
                "lisClient": self._session.bodySampleStoreSession.getClient(self.requestId),
                "accountId": self.accountId,
                "bucket": self._serviceConfig.bodySamplesStorage.bucket,
            }

    def _getImageOriginClientKwargs(self) -> dict | None:
        """Get image origin clients kwargs"""
        if self._session.imageOriginStoreSession:
            return {
                "lisClient": self._session.imageOriginStoreSession.getClient(self.requestId),
                "accountId": self.accountId,
                "bucket": self._serviceConfig.imageOriginStorage.bucket,
            }

    def _getLRSClientKwargs(self) -> dict:
        """Get lrs clients kwargs"""
        return {
            "lrsClient": self._session.remoteSDKSession.getClient(self.requestId),
            "accountId": self.accountId,
        }

    def _getSenderClientKwargs(self) -> dict | None:
        """Get sender clients kwargs"""
        if self._session.senderSession:
            return {
                "senderClient": self._session.senderSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    def _getHandlersClientKwargs(self) -> dict | None:
        """Get handlers clients kwargs"""
        if self._session.handlersSession:
            return {
                "handlersClient": self._session.handlersSession.getClient(self.requestId),
                "accountId": self.accountId,
            }


class BaseStandaloneLambdaRequest(BaseLambdaRequest, ClientsKwargsMixin):
    """Base lambda request"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._luna3Client: _Client = self.app.ctx.luna3session.getClient(self.requestId)


class BaseHandlersRequest(BaseStandaloneLambdaRequest):
    """Base lambda request"""

    @final
    def getRequestData(self, pydanticModel: type[BaseModel]):
        """Get data from request"""
        try:
            return loadDataFromJson(super().json, pydanticModel)
        except VLException as e:
            exc = BaseLambdaException
            exc.errorText = str(e.error.asDict())
            exc.statusCode = 400
            raise exc
